-module(element_number).
% author Vladimir Galunshchikov

-include_lib("nitro/include/nitro.hrl").
-include_lib("nitro/include/event.hrl").

-export([render_element/1]).


render_element(Record) when Record#number.show_if == false ->
  [<<>>];

render_element(Record) ->
  Id = case Record#number.postback of
    [] -> Record#number.id;
    Postback ->
      ID = case Record#number.id of
        [] -> nitro:temp_id();
        I -> I
      end,
      nitro:wire(#event{type=click, postback=Postback, target=ID,
                  source=Record#number.source, delegate=Record#number.delegate }),
      ID
  end,
  
  List = [
    % global
    {<<"accesskey">>, Record#number.accesskey},
    {<<"class">>, Record#number.class},
    
    {<<"contenteditable">>,
      case Record#number.contenteditable of
        true -> "true";
        false -> "false";
        _ -> []
      end},
    
    {<<"contextmenu">>, Record#number.contextmenu},
    
    {<<"dir">>,
      case Record#number.dir of
        "ltr" -> "ltr";
        "rtl" -> "rtl";
        "auto" -> "auto";
        _ -> []
      end},
    
    {<<"draggable">>,
      case Record#number.draggable of
        true -> "true";
        false -> "false";
        _ -> []
      end},
    
    {<<"dropzone">>, Record#number.dropzone},
    
    {<<"hidden">>,
      case Record#number.hidden of
        "hidden" -> "hidden";
        _ -> []
      end},
    
    {<<"id">>, Id},
    {<<"lang">>, Record#number.lang},
    
    {<<"spellcheck">>,
      case Record#number.spellcheck of
        true -> "true";
        false -> "false";
        _ -> []
      end},
    
    {<<"style">>, Record#number.style},
    {<<"tabindex">>, Record#number.tabindex},
    {<<"title">>, Record#number.title},
    
    {<<"translate">>,
      case Record#number.contenteditable of
        "yes" -> "yes";
        "no" -> "no";
        _ -> []
      end},
    
    % spec
    {<<"autocomplete">>,
      case Record#number.autocomplete of
        true -> "on";
        false -> "off";
        _ -> []
      end},
    
    {<<"autofocus">>,
      case Record#number.autofocus of
        true -> "autofocus";
        _ -> []
      end},
    
    {<<"disabled">>,
      case Record#number.disabled of
        true -> "disabled";
        _ -> []
      end},
    
    {<<"form">>, Record#number.form},
    {<<"list">>, Record#number.list},
    {<<"max">>, Record#number.max},
    {<<"min">>, Record#number.min},
    {<<"name">>, Record#number.name},
    {<<"placeholder">>, Record#number.placeholder},
    
    {<<"readonly">>,
      case Record#number.readonly of
        true -> "readonly";
        _ -> []
      end},
    
    {<<"required">>,
      case Record#number.required of
        true -> "required";
        _ -> []
      end},
    
    {<<"step">>, Record#number.step},
    {<<"type">>, <<"number">>},
    {<<"value">>, Record#number.value} | Record#number.data_fields
  ],
  
  wf_tags:emit_tag(<<"input">>, nitro:render(Record#number.body), List).

